@@ -22,12 +22,12 @@
 
 // Qt
 #include <QtCore/QList>
-#include <QtGui/QClipboard>
-#include <QtGui/QCursor>
-#include <QtGui/QGridLayout>
-#include <QtGui/QKeyEvent>
-#include <QtGui/QLabel>
-#include <QtGui/QMenu>
+#include <QClipboard>
+#include <QCursor>
+#include <QGridLayout>
+#include <QKeyEvent>
+#include <QLabel>
+#include <QMenu>
 
 // KDE
 #include <k3bookmarkdrag.h>
@@ -43,6 +43,7 @@
 #include <kmessagebox.h>
 #include <kstandardaction.h>
 #include <kparts/part.h>
+#include <kinputdialog.h>
 
 KBookmarkManager* s_bookmarkManager = 0;
 
@@ -273,9 +274,7 @@ void KonqSidebarBookmarkModule::slotDropped(K3ListView *, QDropEvent *e, Q3ListV
                 parentGroup = bm.toGroup();
             else
                 return;
-        }
-        else if(parent == m_topLevelItem)
-        {
+        } else if(parent == m_topLevelItem) {
             parentGroup = s_bookmarkManager->root();
         }
     } else {
@@ -300,24 +299,30 @@ void KonqSidebarBookmarkModule::slotCreateFolder()
 {
     KonqSidebarBookmarkItem *bi = dynamic_cast<KonqSidebarBookmarkItem*>( tree()->selectedItem() );
     KBookmarkGroup parentGroup;
-    if (bi)
-    {
-	if (bi->bookmark().isGroup())
-	    parentGroup = bi->bookmark().toGroup();
-	else
-	    parentGroup = bi->bookmark().parentGroup();
+    if (bi) {
+        if (bi->bookmark().isGroup()) {
+            parentGroup = bi->bookmark().toGroup();
+        } else {
+            parentGroup = bi->bookmark().parentGroup();
+        }
+    } else if(tree()->selectedItem() == m_topLevelItem) {
+        parentGroup = s_bookmarkManager->root();
     }
-    else if(tree()->selectedItem() == m_topLevelItem)
-    {
-	parentGroup = s_bookmarkManager->root();
+    else {
+        return;
     }
-    else
-	return;
 
-    KBookmark bookmark = parentGroup.createNewFolder("");//TODO use Dialog
-    if(bi && !(bi->bookmark().isGroup()))
-	parentGroup.moveBookmark(bookmark, bi->bookmark());
+    bool ok;
+    const QString str = KInputDialog::getText(i18nc("@title:window", "Create New Bookmark Folder"),
+                                              i18n("New folder:"), QString(), &ok, tree());
+    if (!ok) {
+        return;
+    }
 
+    KBookmark bookmark = parentGroup.createNewFolder(str);
+    if (bi && !(bi->bookmark().isGroup())) {
+        parentGroup.moveBookmark(bookmark, bi->bookmark());
+    }
     s_bookmarkManager->emitChanged( parentGroup );
 }
 
